/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingBusiness;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.ejb.Local;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.logging.Logger;

import jtrackingPersistence.CommandJPA;
import jtrackingPersistence.DeviceJPA;

/**
 * The Class CommandFacadeEJB.
 */
@Stateless
@Local(ICommandBusiness.class)
public class CommandFacadeEJB implements ICommandBusiness {

	/** The em. */
	@PersistenceContext(unitName = "J-trackingJPA")
	private EntityManager em;
	
	/** The Constant log. */
	private static final Logger log = Logger.getLogger(CommandFacadeEJB.class);

	/* (non-Javadoc)
	 * @see jtrackingBusiness.ICommandBusiness#requestBatteyLevel(java.lang.Integer)
	 */
	public Boolean requestBatteyLevel(Integer deviceId) throws Exception {
		
		boolean state = false;

		try {
			DeviceJPA dev = em.find(DeviceJPA.class, deviceId);
			ArrayList<CommandJPA> commands = new ArrayList<>();
			commands.addAll(dev.getCommands());
			// Se sale si hay algún comando igual pendiente de envío
			for (CommandJPA cmd : commands) {
				if (cmd.getCommandTxt().contains("Jradia #get battery level#")) {
					if (cmd.getSent() == false) {
						logInfo("error", "A battery level request pending");
						return state;
					}
				}
			}

			CommandJPA command = new CommandJPA();
			command.setSent(false);
			command.setCommandTxt("Jradia #get battery level#");
			command.setDevice(dev);
			em.persist(command);
			state = true;

		} catch (Exception e) {
			logInfo("error", "A battery level request pending");
		}
		return state;
	}

	/* (non-Javadoc)
	 * @see jtrackingBusiness.ICommandBusiness#requestGsmSignal(java.lang.Integer)
	 */
	public Boolean requestGsmSignal(Integer deviceId) throws Exception {

		boolean state = false;
		try {
			DeviceJPA dev = em.find(DeviceJPA.class, deviceId);
			ArrayList<CommandJPA> commands = new ArrayList<>();
			commands.addAll(dev.getCommands());
			// Se sale si hay algún comando igual pendiente de envío
			for (CommandJPA cmd : commands) {
				if (cmd.getCommandTxt().contains("Jradia #get signal strength#")) {
					if (cmd.getSent() == false) {
						logInfo("error", "A Gsm signal request pending");
						return state;
					}
				}
			}
			CommandJPA command = new CommandJPA();
			command.setSent(false);
			command.setCommandTxt("Jradia #get signal strength#");
			command.setDevice(dev);
			em.persist(command);
			state = true;

		} catch (Exception e) {
			logInfo("error", "A Gsm signal request pending");
		}
		return state;
	}

	/* (non-Javadoc)
	 * @see jtrackingBusiness.ICommandBusiness#checkStatus()
	 */
	@SuppressWarnings("unchecked")
	public Collection<DeviceJPA> checkStatus() throws Exception {
		List<DeviceJPA> devices = null;;
		try {
			Query query = em.createQuery("SELECT d FROM DeviceJPA d ORDER BY id");
			devices = query.getResultList();
		} catch (Exception e) {
			logInfo("error", "Problem checking status");
		}

		return devices;
	}

	/* (non-Javadoc)
	 * @see jtrackingBusiness.ICommandBusiness#stopSendPositions(java.lang.Integer)
	 */
	public Boolean stopSendPositions(Integer deviceId) {
		Boolean state = false;
		try {
			DeviceJPA dev = em.find(DeviceJPA.class, deviceId);
			ArrayList<CommandJPA> commands = new ArrayList<>();
			commands.addAll(dev.getCommands());
			// Se sale si hay algún comando igual pendiente de envío
			for (CommandJPA cmd : commands) {
				if (cmd.getCommandTxt().contains("Jradia #stop sending positions#")) {
					if (cmd.getSent() == false) {
						logInfo("error", "A stop sending command is pending");
						return state;
					}
				}
			}
			CommandJPA command = new CommandJPA();
			command.setSent(false);
			command.setCommandTxt("Jradia #stop sending positions#");
			command.setDevice(dev);
			em.persist(command);
			state = true;

		} catch (Exception e) {
			logInfo("error", "A stop sending command is pending");
		}
		return state;
	}

	/* (non-Javadoc)
	 * @see jtrackingBusiness.ICommandBusiness#deleteCommand(java.lang.Integer)
	 */
	public Boolean deleteCommand(Integer id) throws Exception {

		Boolean state = false;
		
		try {
			CommandJPA cmd = em.find(CommandJPA.class, id);
			em.remove(cmd);
			state = true;
		} catch (Exception e) {
			logInfo("error", "Error Deleting command");
			
		}
		return state;
	}

	/* (non-Javadoc)
	 * @see jtrackingBusiness.ICommandBusiness#startSendPositions(java.lang.Integer)
	 */
	public Boolean startSendPositions(Integer deviceId) throws Exception {
		Boolean state = false;
		try {
			DeviceJPA dev = em.find(DeviceJPA.class, deviceId);
			ArrayList<CommandJPA> commands = new ArrayList<>();
			commands.addAll(dev.getCommands());
			// Se sale si hay algún comando igual pendiente de envío
			for (CommandJPA cmd : commands) {
				if (cmd.getCommandTxt().contains("Jradia #start sending positions#")) {
					if (cmd.getSent() == false) {
						logInfo("error", "A start sending command is pending");
						return state;
					}
				}
			}
			CommandJPA command = new CommandJPA();
			command.setSent(false);
			command.setCommandTxt("Jradia #start sending positions#");
			command.setDevice(dev);
			em.persist(command);
			state = true;

		} catch (Exception e) {
			logInfo("error", "A start sending command is pending");
		}
		return state;
	}

	/* (non-Javadoc)
	 * @see jtrackingBusiness.ICommandBusiness#listCommands()
	 */
    @SuppressWarnings("unchecked")
	public Collection<CommandJPA> listCommands() throws Exception {
    	List<CommandJPA> commands = null;
    	try {
			commands = new ArrayList<>();
			Query query = em.createQuery("FROM CommandJPA");
			commands.addAll(query.getResultList()); 			
		} catch (Exception e) {
			logInfo("error", "Error retriving commands");
		}

		return commands;
    }

	/* (non-Javadoc)
	 * @see jtrackingBusiness.ICommandBusiness#requestStatus(java.lang.Integer)
	 */
	public Boolean requestStatus(Integer deviceId) throws Exception {

		Boolean state = false;
		
		try {
			DeviceJPA dev = em.find(DeviceJPA.class, deviceId);
			ArrayList<CommandJPA> commands = new ArrayList<>();
			commands.addAll(dev.getCommands());
			// Se sale si hay algún comando igual pendiente de envío
			for (CommandJPA cmd : commands) {
				if (cmd.getCommandTxt().contains("Jradia #get status#")) {
					if (cmd.getSent() == false) {
						logInfo("error", "A status command is pendign");
						return state;
					}
				}
			}
			CommandJPA command = new CommandJPA();
			command.setSent(false);
			command.setCommandTxt("Jradia #get status#");
			command.setDevice(dev);
			em.persist(command);
			state = true;

		} catch (Exception e) {
			logInfo("error", "A status command is pendign");
		
		}
		return state;
		
	}

	/**
	 * Instantiates a new command facade EJB.
	 */
	public CommandFacadeEJB() {

	}

	
	protected void logInfo(String type, String message)
	    {
	    	switch (type) {
			case "info" :
				log.info(message);
				break;
			case "error" :
				log.error(message);
				break;
			default:
				break;
			}
	    }

}
