/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingBusiness;

import java.util.Collection;
import java.util.Date;
import javax.ejb.Local;
import javax.persistence.PersistenceException;
import jtrackingPersistence.CommunicationType;
import jtrackingPersistence.DeviceJPA;
import jtrackingPersistence.DeviceModel;
import jtrackingPersistence.MissionJPA;
import jtrackingPersistence.PhoneLineJPA;
import jtrackingPersistence.UserJPA;
import jtrackingPersistence.VehicleJPA;

/**
 * The Interface IManagementBusiness.
 */
@Local
public interface IManagementBusiness {

	/**
	 * Adds the user.
	 *
	 * @param logname the logname
	 * @param encodedPassword the encoded password
	 * @param name the name
	 * @param surname the surname
	 * @param email the email
	 * @param phone the phone
	 * @param rolename the rolename
	 * @return true, if successful
	 * @throws Exception the exception
	 */
	public boolean addUser(String logname, String encodedPassword, String name, String surname, String email,
			String phone, String rolename) throws Exception;

	/**
	 * Gets the users.
	 *
	 * @return the users
	 * @throws Exception the exception
	 */
	public Collection<UserJPA> getUsers() throws Exception;

	/**
	 * Delete user.
	 *
	 * @param logname the logname
	 * @return true, if successful
	 * @throws Exception the exception
	 */
	public boolean deleteUser(String logname) throws Exception;

	/**
	 * Update user.
	 *
	 * @param logname the logname
	 * @param name the name
	 * @param surname the surname
	 * @param email the email
	 * @param phone the phone
	 * @param rolename the rolename
	 * @return true, if successful
	 * @throws Exception the exception
	 */
	public boolean updateUser(String logname, String name, String surname, String email, String phone, String rolename)
			throws Exception;

	/**
	 * Change password.
	 *
	 * @param logname the logname
	 * @param oldEncodedPassword the old encoded password
	 * @param newEncodedPassword the new encoded password
	 * @return true, if successful
	 * @throws Exception the exception
	 */
	public boolean changePassword(String logname, String oldEncodedPassword, String newEncodedPassword)
			throws Exception;

	/**
	 * Adds the mission.
	 *
	 * @param id the id
	 * @param description the description
	 * @param startDate the start date
	 * @param endDate the end date
	 * @return true, if successful
	 * @throws Exception the exception
	 */

	public boolean addMission(String id, String description, Date startDate, Date endDate) throws Exception;

	/**
	 * Gets the missions.
	 *
	 * @return the missions
	 * @throws Exception the exception
	 */
	public Collection<MissionJPA> getMissions() throws Exception;

	/**
	 * Delete mission.
	 *
	 * @param id the id
	 * @return true, if successful
	 * @throws Exception the exception
	 */
	public boolean deleteMission(String id) throws Exception;

	/**
	 * Update mission.
	 *
	 * @param id the id
	 * @param description the description
	 * @param startDate the start date
	 * @param endDate the end date
	 * @return true, if successful
	 * @throws Exception the exception
	 */
	public boolean updateMission(String id, String description, Date startDate, Date endDate) throws Exception;

	/**
	 * Adds the vehicle.
	 *
	 * @param plate the plate
	 * @param brand the brand
	 * @param model the model
	 * @param color the color
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */

	public boolean addVehicle(String plate, String brand, String model, String color) throws PersistenceException;

	/**
	 * Gets the vehicles.
	 *
	 * @return the vehicles
	 * @throws PersistenceException the persistence exception
	 */
	public Collection<VehicleJPA> getVehicles() throws PersistenceException;

	/**
	 * Delete vehicle.
	 *
	 * @param plate the plate
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */
	public boolean deleteVehicle(String plate) throws PersistenceException;

	/**
	 * Update vehicle.
	 *
	 * @param plate the plate
	 * @param brand the brand
	 * @param model the model
	 * @param color the color
	 * @param missionId the mission id
	 * @param deviceId the device id
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */
	public boolean updateVehicle(String plate, String brand, String model, String color, String missionId,
			Integer deviceId) throws PersistenceException;

	/**
	 * Adds the device.
	 *
	 * @param model the model
	 * @param serialNumber the serial number
	 * @param firmwareVersion the firmware version
	 * @param communication the communication
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */
	public boolean addDevice(DeviceModel model, String serialNumber, String firmwareVersion,
			CommunicationType communication) throws PersistenceException;

	/**
	 * Gets the devices.
	 *
	 * @return the devices
	 * @throws PersistenceException the persistence exception
	 */
	public Collection<DeviceJPA> getDevices() throws PersistenceException;

	/**
	 * Delete device.
	 *
	 * @param id the id
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */
	public boolean deleteDevice(Integer id) throws PersistenceException;

	/**
	 * Update device.
	 *
	 * @param deviceId the device id
	 * @param model the model
	 * @param serialNumber the serial number
	 * @param firmwareVersion the firmware version
	 * @param communication the communication
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */
	public boolean updateDevice(Integer deviceId, DeviceModel model, String serialNumber, String firmwareVersion,
			CommunicationType communication) throws PersistenceException;
	
	/**
	 * Adds the phone line.
	 *
	 * @param number the number
	 * @param ICC the icc
	 * @param PIN the pin
	 * @param PUK the puk
	 * @param company the company
	 * @param holder the holder
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */
	public boolean addPhoneLine(String number, String ICC, String PIN, String PUK,
			String company, String holder) throws PersistenceException;
	
	/**
	 * Gets the phone lines.
	 *
	 * @return the phone lines
	 * @throws PersistenceException the persistence exception
	 */
	public Collection<PhoneLineJPA> getPhoneLines() throws PersistenceException;
	
	/**
	 * Update phone line.
	 *
	 * @param number the number
	 * @param ICC the icc
	 * @param PIN the pin
	 * @param PUK the puk
	 * @param company the company
	 * @param holder the holder
	 * @param deviceId the device id
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */
	public boolean updatePhoneLine(String number, String ICC, String PIN, String PUK,
			String company, String holder, Integer deviceId ) throws PersistenceException;
	
	/**
	 * Delete phone line.
	 *
	 * @param nmber the nmber
	 * @return true, if successful
	 * @throws PersistenceException the persistence exception
	 */
	public boolean deletePhoneLine(String nmber) throws PersistenceException;
	
}
