/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingBusiness;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.jboss.logging.Logger;

import jtrackingPersistence.CommunicationType;
import jtrackingPersistence.DeviceJPA;
import jtrackingPersistence.DeviceModel;
import jtrackingPersistence.Jradia01JPA;
import jtrackingPersistence.MissionJPA;
import jtrackingPersistence.PhoneLineJPA;
import jtrackingPersistence.RoleJPA;
import jtrackingPersistence.UserJPA;
import jtrackingPersistence.VehicleJPA;

/**
 * The Class ManagementFacadeEJB.
 */
@Stateless
public class ManagementFacadeEJB implements IManagementBusiness {

	/** The em. */
	@PersistenceContext(unitName = "J-trackingJPA")
	private EntityManager em;
	
	/** The Constant log. */
	private static final Logger log = Logger.getLogger(ManagementFacadeEJB.class);
	

	/*
	 * (non-Javadoc)
	 * 
	 * @see jtrackingBusiness.ImanagementFacadeLocal#addUser(java.lang.String,
	 * java.lang.String, java.lang.String, java.lang.String, java.lang.String,
	 * java.lang.String, java.lang.String)
	 */
	public boolean addUser(String logname, String encodedPassword, String name, String surname, String email,
			String phone, String rolename) throws PersistenceException {

		Boolean state = false;
		
		try {
			UserJPA user = em.find(UserJPA.class, logname);
			if(user == null)
			{
				user = new UserJPA();
				RoleJPA rol = null;
				rol = em.find(RoleJPA.class, rolename);
				user.setLogname(logname);
				user.setPassword(encodedPassword);
				user.setName(name);
				user.setSurname(surname);
				user.setEmail(email);
				user.setPhone(phone);
				user.setRole(rol);
				em.persist(user);
				state = true;
			}
					
		} catch (Exception e) {
			logInfo("error", "Error creating user" + " " + logname);
			throw new PersistenceException();

		}
		if(state != true)
		{
			logInfo("error", "Error creating user" + " " + logname);
		}
		return state;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jtrackingBusiness.ImanagementFacadeLocal#getUsers()
	 */
	@SuppressWarnings("unchecked")
	public Collection<UserJPA> getUsers() throws PersistenceException {
		List<UserJPA> users = null;
		try {
			users = new ArrayList<>();
			Query query = em.createQuery("FROM UserJPA");
			users = query.getResultList();
		} catch (Exception e) {
			logInfo("error", "Error retriving users list");
			
		}

		return users;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#deleteUser(java.lang.String)
	 */
	public boolean deleteUser(String logname) throws PersistenceException {

		Boolean state = false;
		
		try {
			UserJPA user = em.find(UserJPA.class, logname);
			em.remove(user);
			state = true;

		} catch (Exception e) {
			logInfo("error", "Error deleting user" + " " + logname);
			
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#updateUser(java.lang.String,
	 * java.lang.String, java.lang.String, java.lang.String, java.lang.String,
	 * java.lang.String)
	 */
	public boolean updateUser(String logname, String name, String surname, String email, String phone, String rolename)
			throws PersistenceException {

		Boolean state = false;
		
		try {
			UserJPA user = em.find(UserJPA.class, logname);
			RoleJPA rol = null;
			rol = em.find(RoleJPA.class, rolename);
			user.setName(name);
			user.setSurname(surname);
			user.setEmail(email);
			user.setPhone(phone);
			user.setRole(rol);
			state = true;
		} catch (Exception e) {
			logInfo("error", "Error updating user" + " " + logname);
			
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#addMission(java.lang.String,
	 * java.lang.String, java.util.Date, java.util.Date)
	 */
	@Override
	public boolean addMission(String id, String description, Date startDate, Date endDate) throws Exception {

		Boolean state = false;
		
		try {
			MissionJPA mission = em.find(MissionJPA.class, id);
			if(mission == null)
			{
				mission = new MissionJPA();
				mission.setId(id);
				mission.setDescription(description);
				mission.setStartDate(startDate);
				mission.setEndDate(endDate);
				em.persist(mission);
				state = true;

			}
			
		} catch (Exception e) {
			logInfo("error", "Error creating mission" + " " + id);

		}

		if(state != true)
		{
			logInfo("error", "Error creating mission" + " " + id);
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jtrackingBusiness.ImanagementFacadeLocal#getMissions()
	 */
	@SuppressWarnings("unchecked")
	public Collection<MissionJPA> getMissions() throws Exception {
		
		List<MissionJPA> missions = null;
		try {
			missions = new ArrayList<>();
			Query query = em.createQuery("FROM MissionJPA");
			missions = query.getResultList();
		} catch (Exception e) {
			logInfo("error", "Error retriving missions list");

		}
		return missions;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#deleteMission(java.lang.String)
	 */
	@Override
	public boolean deleteMission(String id) throws Exception {

		Boolean state = false;
		
		try {
			MissionJPA mission = em.find(MissionJPA.class, id);
			em.remove(mission);
			state = true;

		} catch (Exception e) {
			logInfo("error", "Error deleting mission" + " " + id);
			
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#updateMission(java.lang.String,
	 * java.lang.String, java.util.Date, java.util.Date)
	 */
	@Override
	public boolean updateMission(String id, String description, Date startDate, Date endDate) throws Exception {

		Boolean state = false;
		
		try {
			MissionJPA mission = em.find(MissionJPA.class, id);
			mission.setDescription(description);
			mission.setStartDate(startDate);
			mission.setEndDate(endDate);
		} catch (Exception e) {
			logInfo("error", "Error updating mission" + " " + id);
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#addVehicle(java.lang.String,
	 * java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public boolean addVehicle(String plate, String brand, String model, String color) throws PersistenceException {

		Boolean state = false;
		
		try {
			VehicleJPA vehicle = em.find(VehicleJPA.class, plate);
			if(vehicle == null)
			{
				vehicle = new VehicleJPA();
				vehicle.setPlate(plate);
				vehicle.setBrand(brand);
				vehicle.setModel(model);
				vehicle.setColor(color);
				em.persist(vehicle);
				state = true;
			}
			
			
		} catch (Exception e) {
			logInfo("error", "Error creating vehicle" + " " + plate);
		}


		if(state != true)
		{
			logInfo("error", "Error creating vehicle" + " " + plate);
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jtrackingBusiness.ImanagementFacadeLocal#getVehiclesJPA()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Collection<VehicleJPA> getVehicles() throws PersistenceException {
		List<VehicleJPA> vehicles = null;
		try {
			vehicles = new ArrayList<>();
			Query query = em.createQuery("FROM VehicleJPA");
			vehicles = query.getResultList();
		} catch (Exception e) {
			logInfo("error", "Error retriving vehicles list");

		}
		return vehicles;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#deleteVehicle(java.lang.String)
	 */
	@Override
	public boolean deleteVehicle(String plate) throws PersistenceException {

		Boolean state = false;
		
		try {
			VehicleJPA vehicle = em.find(VehicleJPA.class, plate);
			em.remove(vehicle);
			state = true;
		} catch (Exception e) {
			logInfo("error", "Error deleting vehicle" + " " + plate);
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#updateVehicle(java.lang.String,
	 * java.lang.String, java.lang.String, java.lang.String, java.lang.String,
	 * java.lang.Integer)
	 */
	@Override
	public boolean updateVehicle(String plate, String brand, String model, String color, String missionId,
			Integer deviceId) throws PersistenceException {

		Boolean state = false;
		
		try {
			VehicleJPA vehicle = em.find(VehicleJPA.class, plate);
			if (missionId == null) {
				vehicle.setMission(null);
			} else {
				MissionJPA mission = em.find(MissionJPA.class, missionId);
				vehicle.setMission(mission);
				mission.getVehicles().add(vehicle);
			}
			if (deviceId == null) {
				vehicle.setDevice(null);

			} else {
				DeviceJPA dev = em.find(DeviceJPA.class, deviceId);
				vehicle.setDevice(dev);
				dev.setVehicle(vehicle);
			}

			vehicle.setBrand(brand);
			vehicle.setModel(model);
			vehicle.setColor(color);
			em.flush();
			state = true;

		} catch (Exception e) {
			logInfo("error", "Error updating vehicle" + " " + plate);
		}

		return state;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jtrackingBusiness.IManagementBusiness#getDevices()
	 */
	@Override
	@SuppressWarnings("unchecked")
	public Collection<DeviceJPA> getDevices() throws PersistenceException {
		List<DeviceJPA> devices = null;
		try {
			devices = new ArrayList<>();
			Query query = em.createQuery("FROM DeviceJPA");
			devices = query.getResultList();
		} catch (Exception e) {
			logInfo("error", "Error retriving devices list");

		}
		return devices;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.IManagementBusiness#deleteDevice(java.lang.Integer)
	 */
	@Override
	public boolean deleteDevice(Integer id) throws PersistenceException {

		Boolean state = false;
		
		try {
			DeviceJPA dev = em.find(DeviceJPA.class, id);
			if(dev.getVehicle() != null) {
				return state;
			}
			else if(dev.getPhoneLine() != null) {
				return state;
			}
			else {
				em.remove(dev);
				state = true;
			}
					

		} catch (Exception e) {
			logInfo("error", "Error deleting device" + " " + id);
		}

		if(state != true)
		{
			logInfo("error", "Error deleting device" + " " + id);
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.ImanagementFacadeLocal#changePassword(java.lang.String,
	 * java.lang.String, java.lang.String)
	 */
	@Override
	public boolean changePassword(String logname, String oldEncodedPassword, String newEncodedPassword)
			throws Exception {
		Boolean state = false;
		try {
			String oldPassword = null;
			UserJPA user = em.find(UserJPA.class, logname);
			if(user != null)
			{
				oldPassword = user.getPassword();
				if (oldEncodedPassword.equals(oldPassword)) {
					user.setPassword(newEncodedPassword);
					state = true;
				}

			}
			
			
		} catch (PersistenceException e) {
			logInfo("error", "Error changing password");
		}

		if(state != true)
		{
			logInfo("error", "Error changing password");
		}
		return state;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.IManagementBusiness#addJradia01(jtrackingPersistence.
	 * DeviceModel, java.lang.String, java.lang.String,
	 * jtrackingPersistence.CommunicationType)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean addDevice(DeviceModel model, String serialNumber, String firmwareVersion,
			CommunicationType communication) throws PersistenceException {
		
		/*La clave primaria es autonúmerica, pero no puede haber dos dispositivos del mismo modelo con
		* el mismo modelo y número de serie. Se declara una constrain unique para model y serielNumber
		* ver DeviceJpa.java, por lo que hay que ver si hay un dispositivo con esos valores antes de crearlo
		*/
	
		List<DeviceJPA> devs; 
		Boolean state = false;
		
		try {
			devs = new ArrayList<>();
			Query query = em.createQuery("SELECT d FROM DeviceJPA d WHERE d.model = :model AND d.serialNumber = :serial");
			query.setParameter("model", model);
			query.setParameter("serial", serialNumber);
			devs.addAll(query.getResultList());
			if(devs.isEmpty())
			{
				if(model == DeviceModel.JRADIA01)
				{
					Jradia01JPA jRadia = new Jradia01JPA(model, serialNumber, firmwareVersion, communication);
					em.persist(jRadia);
					state = true;
				}
			}
				
			
		} catch (PersistenceException e) {
			logInfo("error", "Error creating device" + " " + serialNumber);
		}

		if(state != true)
		{
			logInfo("error", "Error creating device" + " " + serialNumber);
		}
		return state;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.IManagementBusiness#updateJradia01(jtrackingPersistence
	 * .DeviceModel, java.lang.String, java.lang.String,
	 * jtrackingPersistence.CommunicationType, java.lang.Long, java.lang.Long,
	 * java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean updateDevice(Integer deviceId, DeviceModel model, String serialNumber, String firmwareVersion,
			CommunicationType communication) throws PersistenceException {

		Boolean state = false;
		List<DeviceJPA> devs = new ArrayList<>(); 
		
		/*
		 * Hay que tener cuidado con la constraint de clave alternativa model+serialnumber
		 * Se busca si hay un objeto con la misma clave alternativa, si no lo hay o si el encontrado
		 * es el queremos modificar permitimos la modificación
		 * Si el encontrado no es el mismo, no lo permite
		 * Ej queremos modificar    1 JRADIA01 001 por 1 JRADIA01 002  el dispositivo JRADIA01 002 
		 * ya existe, luego no lo permite. 
		 * 
		 */

		try {
			Query query = em.createQuery("SELECT d FROM DeviceJPA d WHERE d.model = :model AND d.serialNumber = :serial");
			query.setParameter("model", model);
			query.setParameter("serial", serialNumber);
			devs.addAll(query.getResultList());
			
			DeviceJPA dev = em.find(DeviceJPA.class, deviceId);
			
			if((devs.isEmpty()) || ((devs.get(0).getModel().equals(dev.getModel()) && (devs.get(0).getSerialNumber().equals(dev.getSerialNumber())))))
				{
				dev.setModel(model);
				dev.setSerialNumber(serialNumber);
				dev.setFirmwareVersion(firmwareVersion);
				dev.setCommunication(communication);
				em.flush();
				state = true;
				}
			
		} catch (Exception e) {
			e.printStackTrace();
			logInfo("error", "Error updating device" + " " + serialNumber);
		}
		if(state != true)
		{
			logInfo("error", "Error updating device" + " " + serialNumber);
		}
		
		return state;
	}

	/**
	 * Instantiates a new management facade EJB.
	 */
	public ManagementFacadeEJB() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jtrackingBusiness.IManagementBusiness#addPhoneLine(java.lang.String,
	 * java.lang.String, java.lang.String, java.lang.String, java.lang.String,
	 * java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean addPhoneLine(String number, String ICC, String PIN, String PUK, String company, String holder)
			throws PersistenceException {
		/*
		 * El ICC es clave alternativa no puede haber dos ICCs iguales
		 */
		
		Boolean state = false;
		List<PhoneLineJPA> lines; 
		
		try {
			lines = new ArrayList<>();
			Query query = em.createQuery("SELECT l FROM PhoneLineJPA l WHERE l.ICC = :ICC");
			query.setParameter("ICC", ICC);
			lines.addAll(query.getResultList());
			
			if(lines.isEmpty())
			{
				PhoneLineJPA pline = em.find(PhoneLineJPA.class, number);
				if(pline == null)
				{
					pline = new PhoneLineJPA();
					pline.setPhoneNumber(number);
					pline.setICC(ICC);
					pline.setPIN(PIN);
					pline.setPUK(PUK);
					pline.setPUK(PUK);
					pline.setCompany(company);
					pline.setHolder(holder);
					em.persist(pline);
					state = true;
				}
			}
			
			
			
		} catch (Exception e) {
			logInfo("error", "Error creating phone line" + " " + number);
		}
		if(state != true)
		{
			logInfo("error", "Error creating phone line" + " " + number);
		}
		return state;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.IManagementBusiness#getPhoneLines(java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Collection<PhoneLineJPA> getPhoneLines() throws PersistenceException {

		List<PhoneLineJPA> phoneLines = null;
		try {
			phoneLines = new ArrayList<>();
			Query query = em.createQuery("FROM PhoneLineJPA");
			phoneLines = query.getResultList();
		} catch (Exception e) {
			logInfo("error", "Error getting phone lines");

		}
		return phoneLines;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.IManagementBusiness#updatePhoneLine(java.lang.String,
	 * java.lang.String, java.lang.String, java.lang.String, java.lang.String,
	 * java.lang.String, java.lang.Integer)
	 */
	@Override
	public boolean updatePhoneLine(String number, String ICC, String PIN, String PUK, String company, String holder,
			Integer deviceId) throws PersistenceException {
		
		Boolean state = false;
		
		
		try {
				PhoneLineJPA pline = em.find(PhoneLineJPA.class, number);
				if (deviceId == null) {
					pline.setDevice(null);
				} else {
					DeviceJPA dev = em.find(DeviceJPA.class, deviceId);
					pline.setDevice(dev);
					dev.setPhoneLine(pline);
				}
				
				pline.setICC(ICC);
				pline.setPIN(PIN);
				pline.setPUK(PUK);
				pline.setCompany(company);
				pline.setHolder(holder);
				em.flush();
				state = true;
			
		} catch (Exception e) {
			logInfo("error", "Error updating phone line" + " " + number);
		}
		return state;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jtrackingBusiness.IManagementBusiness#deletePhoneLine(java.lang.String)
	 */
	@Override
	public boolean deletePhoneLine(String number) throws PersistenceException {

		Boolean state = false;
		
		try {
			PhoneLineJPA pline = em.find(PhoneLineJPA.class, number);
			em.remove(pline);
			state = true;
			
		} catch (Exception e) {
			logInfo("error", "Error deleting phone line" + " " + number);
		}

		return state;
	}

	protected void logInfo(String type, String message)
    {
    	switch (type) {
		case "info" :
			log.info(message);
			break;
		case "error" :
			log.error(message);
			break;
		default:
			break;
		}
    }

}
