/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingBusiness;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.jboss.logging.Logger;
import jtrackingPersistence.PositionJPA;

/**
 * The Class MappingFacadeEJB.
 */
@Stateless
public class MappingFacadeEJB implements IMappingBusiness {


	/** The em. */
	@PersistenceContext(unitName = "J-trackingJPA")
	private EntityManager em;
	
	/** The Constant log. */
	private static final Logger log = Logger.getLogger(MappingFacadeEJB.class);

	/* (non-Javadoc)
	 * @see jtrackingBusiness.IMappingBusiness#getPositions(java.lang.String, java.util.Date, java.util.Date)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Collection<PositionJPA> getPositions(String plate, Date startDate, Date endDate) {
		List<PositionJPA> positions = null;
		try {
			positions = new ArrayList<>();
			Query query  = em.createQuery("SELECT p FROM  PositionJPA p WHERE p.vehicle.plate = :plate AND p.positionTimeStamp BETWEEN :startDate AND :endDate ORDER BY p.positionTimeStamp asc");
			query.setParameter("plate", plate);
			query.setParameter("startDate", startDate);
			query.setParameter("endDate", endDate);
			positions.addAll(query.getResultList());
		} catch (Exception e) {
			logInfo("error", "Error getting positions for vehicle" + " " + plate);
		}
		return positions;
	}

	/* (non-Javadoc)
	 * @see jtrackingBusiness.IMappingBusiness#getRealTime(java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Collection<PositionJPA> getRealTime(String plate) {
		List<PositionJPA> positions = null;
		try {
			positions = new ArrayList<>();
			Query query  = em.createQuery("SELECT p FROM  PositionJPA p WHERE p.vehicle.plate = :plate ORDER BY p.positionTimeStamp desc").setMaxResults(5);
			query.setParameter("plate", plate);
			
			positions.addAll(query.getResultList());
		} catch (Exception e) {
			logInfo("error", "Error getting real time positions for vehicle" + " " + plate);
		}
		return positions;
	}


    /**
     * Instantiates a new mapping facade EJB.
     */
    public MappingFacadeEJB() {
     
    }

    protected void logInfo(String type, String message)
    {
    	switch (type) {
		case "info" :
			log.info(message);
			break;
		case "error" :
			log.error(message);
			break;
		default:
			break;
		}
    }
    
}
