/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingBusiness;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.jboss.logging.Logger;
import jtrackingPersistence.CommandJPA;
import jtrackingPersistence.DeviceJPA;
import jtrackingPersistence.DeviceModel;
import jtrackingPersistence.GpsState;
import jtrackingPersistence.PositionJPA;
import jtrackingPersistence.VehicleJPA;

/**
 * The Class ReceptionFacadeEJB.
 */
@Stateless
@WebService
public class ReceptionFacadeEJB implements IReceptionBusiness {

	/** The em. */
	@PersistenceContext(unitName = "J-trackingJPA")
	private EntityManager em;

	/** The Constant log. */
	private static final Logger log = Logger.getLogger(ReceptionFacadeEJB.class);
	
	/**
	 * Instantiates a new reception facade EJB.
	 */
    public ReceptionFacadeEJB() {
        
    }

	/* (non-Javadoc)
	 * @see jtrackingBusiness.IReceptionBusiness#receivePosition(java.lang.String)
	 */
    @SuppressWarnings("deprecation")
	@WebMethod
     public boolean receivePosition(String position) {
		  	
    	String[] matrix;
		ArrayList<String> lista = new ArrayList<>();
		DeviceJPA dev;
		VehicleJPA veh;
		Boolean state = false;
		
		//Comprobamos si procede de un dispositivo y contine la cadena position;
		if(position.contains("Jradia") && position.contains("position:"))
		{
			
				try {
					/*
					 * parseamos todo
					Jradia-01-001 position:  2017/10/08,19:43:44,40.4734624,N,003.3739540,W,500,000.21,227.43,2
					
					
					Contenido de la lista
					=====================
					
					0---Jradia		IDENTIFICADOR DEL SISTEMA
					1---01			MODELO
					2---001			NÚMERO DE SERIE
					3---position:	TIPO DE INFO
					4---2017/09/26 	FECHA
					5---16:57:41	HORA
					6---40.4775472  LATITUD
					7---N			N-S
					8---003.3701228 LONGITUD
					9---W			E-W
					10---498		ALTITUD
					11---034.66		SPEED
					12---037.43		RUMBO NO UTILIZADO.
					13---2			GPSSTATUS
					
					AT^SGPSR=<DataType>
					Response(s)
					If <DataType>=0:
					^SGPSR: [<GpsDate>, <GpsUTCTime>, <GpsLatitude>, <N-S-Indicator>, <GpsLongitude>, <E-
					W-Indicator>, <GpsAltitude>, <GpsSpeed>, <GpsCourse>, <GpsStatus>]
					OK
					<GpsStatus> (num)
					0 Not fix
					1 Reserved for future use
					2 2D-fix
					3 3D-fix
					4 Reserved for future use
					5 Reserved for future use
					*/

										
					position = position.replaceAll(" |-", ",");
					position = position.replaceAll("position:,", "position:");
					matrix = position.split(",");
					lista.addAll(Arrays.asList(matrix));
														
					
					Date timeStamp = new Date(lista.get(4) + " " + lista.get(5));
					
					Double latitude = (lista.get(7).equals("N")? 
							Double.parseDouble(lista.get(6)):Double.parseDouble(lista.get(6))*-1);
					

					Double longitude = (lista.get(9).equals("E")? 
							Double.parseDouble(lista.get(8)):Double.parseDouble(lista.get(8))*-1);
					
								
					Float altitude = Float.parseFloat(lista.get(10));
					Float speed = Float.parseFloat(lista.get(11));
					GpsState gpsState = null;
					switch (lista.get(13)) {
					
					case "2":
						gpsState = GpsState.TWO_D;
						break;
					case "3":
						gpsState = GpsState.THREE_D;
						break;

					default:
						//GPS no válido descartar la sentencia y salir
						return false;
					}
					
					dev = getDevice(lista);
					veh = dev.getVehicle();
					if(veh != null) 
					{
						PositionJPA pos = new PositionJPA();
						pos.setVehicle(veh);
						pos.setAltitude(altitude);;
						pos.setGps(gpsState);
						pos.setLatitude(latitude);
						pos.setLongitude(longitude);
						pos.setPositionTimeStamp(timeStamp);
						pos.setSpeed(speed);
						em.persist(pos);	
						state = true;
					}
					
					} catch (Exception e) {
					
						logInfo("error", "Error receiving position" + " " + position);
					
			}
		 				
		}
		if(state != true)
		{
			logInfo("error", "Error receiving position" + " " + position);
		}
		return state;
		
		
		
		
    }

	/* (non-Javadoc)
	 * @see jtrackingBusiness.IReceptionBusiness#obtainConfiguration(java.lang.String)
	 */
    @WebMethod
    public Collection<String> obtainConfiguration(String deviceId) {
    	
    	ArrayList<CommandJPA> comandos = new ArrayList<>();
		ArrayList<String> comandosTxt = new ArrayList<>();
		String[] matrix;
		ArrayList<String> lista = new ArrayList<>();
		DeviceJPA dev = null;
		
		if(deviceId.contains("Jradia"))
		{

			try {
				deviceId = deviceId.replace("-", ",");
				matrix = deviceId.split(",");
				lista.addAll(Arrays.asList(matrix));
				dev = getDevice(lista);
				comandos.addAll(dev.getCommands());	
				for(CommandJPA cmd: comandos)
				{
					if(cmd.getSent() == false)
					{
						comandosTxt.add(cmd.getCommandTxt());
						cmd.setSent(true);
					}
					
					
				}
					} catch (Exception e) {					
						logInfo("error", "Error getting commands" + " " + deviceId);							
			}
			
		}
		else
		{
			logInfo("error", "Error getting commands" + " " + deviceId);							
		}
		return comandosTxt;	
		
    }


	/* (non-Javadoc)
	 * @see jtrackingBusiness.IReceptionBusiness#receiveGsmSignal(java.lang.String)
	 */
    @WebMethod
    public boolean receiveGsmSignal(String signal) {
    	
    	String[] matrix;
		ArrayList<String> lista = new ArrayList<>();
		StringBuilder sb1 = new StringBuilder();
		DeviceJPA dev = null;
		Boolean state = false;
		
		//Comprobamos si procede de un dispositivo y contine la cadenastrength;
		if(signal.contains("Jradia") && signal.contains("strength:"))
		{
			
				try {
					//parseamos todo
										
					signal = signal.replaceAll(" |-", ",");
					signal = signal.replaceAll(",strength:,", ",strength:");
					matrix = signal.split(",");
					lista.addAll(Arrays.asList(matrix));
					
					sb1.append(lista.get(6));
					
					//recuperamos el dispotivo
					dev = getDevice(lista);
					if(dev != null)
					{
						dev.setGsmSignal(-(Float.parseFloat(sb1.toString())));
						return true;
					}
								
					} catch (Exception e) {
					
						logInfo("error", "Error receiving GSM signal" + " " + signal);
				}
		 				
		}
		if(state != true)
		{
			logInfo("error", "Error receiving GSM signal" + " " + signal);
		}
		return state;	
    }

	/* (non-Javadoc)
	 * @see jtrackingBusiness.IReceptionBusiness#receiveBatteryLevel(java.lang.String)
	 */
    @WebMethod
    public boolean receiveBatteryLevel(String level) {
    	String[] matrix;
		ArrayList<String> lista = new ArrayList<>();
		StringBuilder sb1 = new StringBuilder();
		DeviceJPA dev = null;
		Boolean state = false;
		
		//Comprobamos si procede de un dispositivo y contine la cadena battery;
		if(level.contains("Jradia") && level.contains("battery:"))
		{
			
				try {
					//parseamos todo
					level = level.replaceAll("  | |-", ",");
					matrix = level.split(",");
					lista.addAll(Arrays.asList(matrix));
					
					sb1.append(lista.get(4));
					
					//Recuperamos el dispositivo
					dev = getDevice(lista);
					if(dev != null)
					{
						dev.setBatteryLevel(Float.parseFloat(sb1.toString()));
						state = true;
					}
					
					} catch (Exception e) {
					
						logInfo("error", "Error receiving battery level" + " " + level);
			}
		 				
		}
		if(state != true)
		{
			logInfo("error", "Error receiving battery level" + " " + level);
		}
		return state;
		
		
    }

	/* (non-Javadoc)
	 * @see jtrackingBusiness.IReceptionBusiness#receiveStatus(java.lang.String)
	 */
    @WebMethod
    public boolean receiveStatus(String status) {
    	String[] matrix;
		ArrayList<String> lista = new ArrayList<>();
		StringBuilder sb1 = new StringBuilder();
		DeviceJPA dev = null;
		Boolean state = false;
		
		//Comprobamos si procede de un dispositivo y contine la cadena status;
		if(status.contains("Jradia") && status.contains("status:"))
		{
			
				try {
					//parseamos todo
					status = status.replaceAll("-|, ", ",");
					status = status.replaceAll(" status: ", ",status:,");
					matrix = status.split(",");
					lista.addAll(Arrays.asList(matrix));
					sb1.append(lista.get(4)).append(" ").append(lista.get(5)).append(" ").append(lista.get(6)).append(" ").append(lista.get(7));
					
					//recuperamos el dispositivo
					dev = getDevice(lista);
					if(dev != null)
					{
						dev.setStatus(sb1.toString());
						state = true;
					}
					
					} catch (Exception e) {
					
						logInfo("error", "Error receiving status" + " " + status);				
				}						
		}
		if(state != true)
		{
			logInfo("error", "Error receiving status" + " " + status);
		}
		return state;
    }

    /**
     * Gets the device.
     *
     * @param lista the lista
     * @return the device
     */
    @WebMethod(exclude = true)
    protected DeviceJPA getDevice(List<String> lista)
    {
    	DeviceJPA dev = null;
    	StringBuilder sb = new StringBuilder();
    	try {
			sb.append(lista.get(0).toUpperCase()).append(lista.get(1));
			Query query  = em.createQuery("SELECT d FROM  DeviceJPA d WHERE d.model = :model AND d.serialNumber = :serialNumber");
			if(sb.toString().equals("JRADIA01"))
			{
				query.setParameter("model", DeviceModel.JRADIA01);
			}
			query.setParameter("serialNumber", lista.get(2));
			dev = (DeviceJPA) query.getSingleResult();
		} catch (Exception e) {
			
		}
		return dev;
    }
    
    @WebMethod(exclude = true)
    protected void logInfo(String type, String message)
    {
    	switch (type) {
		case "info" :
			log.info(message);
			break;
		case "error" :
			log.error(message);
			break;
		default:
			break;
		}
    }

}
