/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPersistence;

import java.io.Serializable;
import java.lang.String;
import java.util.Collection;

import javax.persistence.*;


/**
 * The Class DeviceJPA.
 */
@Entity
@Inheritance(strategy=InheritanceType.SINGLE_TABLE)
@Table(name = "jtracking.devices",
		uniqueConstraints=@UniqueConstraint(columnNames={"model", "serialnumber"}))
public abstract class DeviceJPA implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -6468968772622891766L;

	/** The id. */
	@Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	private Integer id;
	
	/** The model. */
	@Column(nullable=false, length=15)
	@Enumerated(EnumType.STRING)
	private DeviceModel model;
	
	/** The serial number. */
	@Column(nullable=false, length=15)
	private String serialNumber;
	
	/** The firmware version. */
	@Column(nullable=false, length=20)
	private String firmwareVersion;
	
	/** The communication. */
	@Column(nullable=false)
	@Enumerated(EnumType.STRING)
	private CommunicationType communication;
	
	/** The battery level. */
	@Column(nullable=false)
	protected Float batteryLevel;
		
	/** The gsm signal. */
	@Column(nullable=false)
	protected Float gsmSignal;
	
	/** The status. */
	@Column(length=180, nullable=false)
	protected String status;
	
	/** The phone line. */
	@OneToOne(mappedBy="device")
	private PhoneLineJPA phoneLine;
	
	/** The vehicle. */
	@OneToOne(mappedBy="device")
	private VehicleJPA vehicle;
	
	/** The commands. */
	@OneToMany(mappedBy="device", cascade=CascadeType.REMOVE)
	private Collection<CommandJPA> commands;
	

	/**
	 * Gets the id.
	 *
	 * @return the id
	 */
	public Integer getId() {
		return id;
	}



	/**
	 * Sets the id.
	 *
	 * @param id the new id
	 */
	public void setId(Integer id) {
		this.id = id;
	}



	/**
	 * Gets the model.
	 *
	 * @return the model
	 */
	public DeviceModel getModel() {
		return model;
	}



	/**
	 * Sets the model.
	 *
	 * @param model the new model
	 */
	public void setModel(DeviceModel model) {
		this.model = model;
	}



	/**
	 * Gets the serial number.
	 *
	 * @return the serial number
	 */
	public String getSerialNumber() {
		return serialNumber;
	}



	/**
	 * Sets the serial number.
	 *
	 * @param serialNumber the new serial number
	 */
	public void setSerialNumber(String serialNumber) {
		this.serialNumber = serialNumber;
	}



	/**
	 * Gets the firmware version.
	 *
	 * @return the firmware version
	 */
	public String getFirmwareVersion() {
		return firmwareVersion;
	}



	/**
	 * Sets the firmware version.
	 *
	 * @param firmwareVersion the new firmware version
	 */
	public void setFirmwareVersion(String firmwareVersion) {
		this.firmwareVersion = firmwareVersion;
	}



	/**
	 * Gets the communication.
	 *
	 * @return the communication
	 */
	public CommunicationType getCommunication() {
		return communication;
	}


	/**
	 * Sets the communication.
	 *
	 * @param communication the new communication
	 */
	public void setCommunication(CommunicationType communication) {
		this.communication = communication;
	}

	/**
	 * Gets the battery level.
	 *
	 * @return the battery level
	 */
	public Float getBatteryLevel() {
		return batteryLevel;
	}

	/**
	 * Sets the battery level.
	 *
	 * @param batteryLevel the new battery level
	 */
	public void setBatteryLevel(Float batteryLevel) {
		this.batteryLevel = batteryLevel;
	}

	/**
	 * Gets the gsm signal.
	 *
	 * @return the gsm signal
	 */
	public Float getGsmSignal() {
		return gsmSignal;
	}

	/**
	 * Sets the gsm signal.
	 *
	 * @param gsmSignal the new gsm signal
	 */
	public void setGsmSignal(Float gsmSignal) {
		this.gsmSignal = gsmSignal;
	}

	/**
	 * Gets the status.
	 *
	 * @return the status
	 */
	public String getStatus() {
		return status;
	}

	/**
	 * Sets the status.
	 *
	 * @param status the new status
	 */
	public void setStatus(String status) {
		this.status = status;
	}
	
	/**
	 * Gets the phone line.
	 *
	 * @return the phone line
	 */
	public PhoneLineJPA getPhoneLine() {
		return phoneLine;
	}



	/**
	 * Sets the phone line.
	 *
	 * @param phoneLine the new phone line
	 */
	public void setPhoneLine(PhoneLineJPA phoneLine) {
		this.phoneLine = phoneLine;
	}



	/**
	 * Gets the vehicle.
	 *
	 * @return the vehicle
	 */
	public VehicleJPA getVehicle() {
		return vehicle;
	}



	/**
	 * Sets the vehicle.
	 *
	 * @param vehicle the new vehicle
	 */
	public void setVehicle(VehicleJPA vehicle) {
		this.vehicle = vehicle;
	}

	

	/**
	 * Gets the commands.
	 *
	 * @return the commands
	 */
	public Collection<CommandJPA> getCommands() {
		return commands;
	}



	/**
	 * Sets the commands.
	 *
	 * @param commands the new commands
	 */
	public void setCommands(Collection<CommandJPA> commands) {
		this.commands = commands;
	}




	/**
	 * Instantiates a new device JPA.
	 */
	public DeviceJPA() {
		super();
	}



	/**
	 * Instantiates a new device JPA.
	 *
	 * @param model the model
	 * @param serialNumber the serial number
	 * @param firmwareVersion the firmware version
	 * @param communication the communication
	 */
	public DeviceJPA(DeviceModel model, String serialNumber, String firmwareVersion, CommunicationType communication) {
		super();
		this.model = model;
		this.serialNumber = serialNumber;
		this.firmwareVersion = firmwareVersion;
		this.communication = communication;
		this.batteryLevel = 0.0F;
		this.gsmSignal = 0.0F;
		this.status = "Not available";
	}  
	
	
	
	}
