/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPersistence;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * The Class PositionJPA.
 */
@Entity
@Table(name = "jtracking.positions", 
indexes = { @Index(name="positionIndex", columnList=("vehicle_fk, positiontimestamp"))})

public class PositionJPA implements Serializable {

	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 2387347355464113345L;

	/** The id. */
	@Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	private Integer id;
	
	/** The position time stamp. */
	@Column(nullable=false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date positionTimeStamp;
	
	/** The latitude. */
	@Column(nullable = false)
	private Double latitude;
	
	/** The longitude. */
	@Column(nullable = false)
	private Double longitude;
	
	
	/** The altitude. */
	private Float altitude;
	
	/** The Speed. */
	@Column(nullable = false)
	private Float Speed;
	
	/** The gps. */
	@Column(nullable = false)
	@Enumerated(EnumType.STRING)
	private GpsState gps;
	
	/** The vehicle. */
	@ManyToOne
	@JoinColumn(name="vehicle_fk")
	private VehicleJPA vehicle;
		
	/**
	 * Gets the id.
	 *
	 * @return the id
	 */
	public Integer getId() {
		return this.id;
	}

	/**
	 * Sets the id.
	 *
	 * @param id the new id
	 */
	public void setId(Integer id) {
		this.id = id;
	}   
	
	/**
	 * Gets the position time stamp.
	 *
	 * @return the position time stamp
	 */
	public Date getPositionTimeStamp() {
		return positionTimeStamp;
	}

	/**
	 * Sets the position time stamp.
	 *
	 * @param positionTimeStamp the new position time stamp
	 */
	public void setPositionTimeStamp(Date positionTimeStamp) {
		this.positionTimeStamp = positionTimeStamp;
	}
	
	/**
	 * Gets the latitude.
	 *
	 * @return the latitude
	 */
	public Double getLatitude() {
		return this.latitude;
	}

	/**
	 * Sets the latitude.
	 *
	 * @param latitude the new latitude
	 */
	public void setLatitude(Double latitude) {
		this.latitude = latitude;
	}   
	
	/**
	 * Gets the longitude.
	 *
	 * @return the longitude
	 */
	public Double getLongitude() {
		return this.longitude;
	}

	/**
	 * Sets the longitude.
	 *
	 * @param longitude the new longitude
	 */
	public void setLongitude(Double longitude) {
		this.longitude = longitude;
	}   
	
	  
	
	/**
	 * Gets the altitude.
	 *
	 * @return the altitude
	 */
	public Float getAltitude() {
		return this.altitude;
	}

	/**
	 * Sets the altitude.
	 *
	 * @param altitude the new altitude
	 */
	public void setAltitude(Float altitude) {
		this.altitude = altitude;
	}   
	
	/**
	 * Gets the speed.
	 *
	 * @return the speed
	 */
	public Float getSpeed() {
		return this.Speed;
	}

	/**
	 * Sets the speed.
	 *
	 * @param Speed the new speed
	 */
	public void setSpeed(Float Speed) {
		this.Speed = Speed;
	}   
	
	/**
	 * Gets the gps.
	 *
	 * @return the gps
	 */
	public GpsState getGps() {
		return this.gps;
	}

	/**
	 * Sets the gps.
	 *
	 * @param gps the new gps
	 */
	public void setGps(GpsState gps) {
		this.gps = gps;
	}
	

	/**
	 * Gets the vehicle.
	 *
	 * @return the vehicle
	 */
	public VehicleJPA getVehicle() {
		return vehicle;
	}

	/**
	 * Sets the vehicle.
	 *
	 * @param vehicle the new vehicle
	 */
	public void setVehicle(VehicleJPA vehicle) {
		this.vehicle = vehicle;
	}

	/**
	 * Instantiates a new position JPA.
	 */
	public PositionJPA() {
		super();
	}   
   
	
}
