/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPresentation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import javax.annotation.PostConstruct;
import javax.enterprise.context.RequestScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import jtrackingBusiness.IManagementBusiness;
import jtrackingPersistence.CommunicationType;
import jtrackingPersistence.DeviceModel;

/**
 * The Class AddDeviceBean.
 */
@Named("device")
@RequestScoped
public class AddDeviceBean{

	/** The management. */
	@Inject
	private IManagementBusiness management;

	/** The model. */
	private DeviceModel model;

	/** The serial number. */
	private String serialNumber;

	/** The firmware version. */
	private String firmwareVersion;

	/** The communication type. */
	private CommunicationType communicationType;

	/** The communications types collection. */
	private Collection<CommunicationType> communicationsTypesCollection = new ArrayList<>();
	
	/** The device model collection. */
	private Collection<DeviceModel> deviceModelCollection = new ArrayList<>();

	/**
	 * Gets the model.
	 *
	 * @return the model
	 */
	public DeviceModel getModel() {
		return model;
	}

	/**
	 * Sets the model.
	 *
	 * @param model the new model
	 */
	public void setModel(DeviceModel model) {
		this.model = model;
	}

	/**
	 * Gets the serial number.
	 *
	 * @return the serial number
	 */
	public String getSerialNumber() {
		return serialNumber;
	}

	/**
	 * Sets the serial number.
	 *
	 * @param serialNumber the new serial number
	 */
	public void setSerialNumber(String serialNumber) {
		this.serialNumber = serialNumber;
	}

	/**
	 * Gets the firmware version.
	 *
	 * @return the firmware version
	 */
	public String getFirmwareVersion() {
		return firmwareVersion;
	}

	/**
	 * Sets the firmware version.
	 *
	 * @param firmwareVersion the new firmware version
	 */
	public void setFirmwareVersion(String firmwareVersion) {
		this.firmwareVersion = firmwareVersion;
	}

	/**
	 * Gets the communication type.
	 *
	 * @return the communication type
	 */
	public CommunicationType getCommunicationType() {
		return communicationType;
	}

	/**
	 * Sets the communication type.
	 *
	 * @param communicationType the new communication type
	 */
	public void setCommunicationType(CommunicationType communicationType) {
		this.communicationType = communicationType;
	}

	/**
	 * Gets the communications types collection.
	 *
	 * @return the communications types collection
	 */
	public Collection<CommunicationType> getCommunicationsTypesCollection() {
		return communicationsTypesCollection;
	}

	/**
	 * Sets the communications types collection.
	 *
	 * @param communicationsTypesCollection the new communications types collection
	 */
	public void setCommunicationsTypesCollection(Collection<CommunicationType> communicationsTypesCollection) {
		this.communicationsTypesCollection = communicationsTypesCollection;
	}

	/**
	 * Gets the device model collection.
	 *
	 * @return the device model collection
	 */
	public Collection<DeviceModel> getDeviceModelCollection() {
		return deviceModelCollection;
	}

	/**
	 * Sets the device model collection.
	 *
	 * @param deviceModelCollection the new device model collection
	 */
	public void setDeviceModelCollection(Collection<DeviceModel> deviceModelCollection) {
		this.deviceModelCollection = deviceModelCollection;
	}

	/**
	 * Inits the.
	 */
	@PostConstruct
	public void init() {

		communicationsTypesCollection.addAll(Arrays.asList(CommunicationType.values()));
		deviceModelCollection.addAll(Arrays.asList(DeviceModel.values()));
	}

	/**
	 * Adds the device.
	 */
	public void addDevice() {

		try {
			if (management.addDevice(model, serialNumber, firmwareVersion, communicationType)) {
				StringBuilder message = new StringBuilder("Sucess: \n device ").append(model).append(" ")
						.append(serialNumber).append(" added");
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null, new FacesMessage(message.toString(), null));
			} else {
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error creating Jradia01", null));
			}
		} catch (Exception e) {

			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error while adding Jradia", null));

		}
	}

	/**
	 * Instantiates a new adds the device bean.
	 */
	public AddDeviceBean() {

	}

}
