/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPresentation;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import javax.annotation.PostConstruct;
import javax.enterprise.context.SessionScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import jtrackingBusiness.ICommandBusiness;
import jtrackingPersistence.CommandJPA;

/**
 * The Class ListCommandsBeans.
 */
@Named("commands")
@SessionScoped
public class ListCommandsBeans implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 8061044313715625688L;

	/** The commander. */
	@Inject
	private ICommandBusiness commander;

	/** The commands collection. */
	private Collection<CommandJPA> commandsCollection = new ArrayList<>();

	/** The command id. */
	private Integer commandId;

	/** The command. */
	private CommandJPA command;

	/**
	 * Gets the command id.
	 *
	 * @return the command id
	 */
	public Integer getCommandId() {
		return commandId;
	}

	/**
	 * Sets the command id.
	 *
	 * @param commandId the new command id
	 */
	public void setCommandId(Integer commandId) {
		this.commandId = commandId;
	}

	/**
	 * Gets the command.
	 *
	 * @return the command
	 */
	public CommandJPA getCommand() {
		return command;
	}

	/**
	 * Sets the command.
	 *
	 * @param command the new command
	 */
	public void setCommand(CommandJPA command) {
		this.command = command;
	}

	/**
	 * Gets the commands collection.
	 *
	 * @return the commands collection
	 */
	public Collection<CommandJPA> getCommandsCollection() {
		return commandsCollection;
	}

	/**
	 * Sets the commands collection.
	 *
	 * @param commandsCollection the new commands collection
	 */
	public void setCommandsCollection(Collection<CommandJPA> commandsCollection) {
		this.commandsCollection = commandsCollection;
	}

	/**
	 * Retrieve commands list.
	 *
	 * @return the string
	 */
	public String retrieveCommandsList() {

		try {
			commandsCollection.clear();
			commandsCollection.addAll(commander.listCommands());
			FacesContext.getCurrentInstance().getExternalContext().redirect("commands.xhtml");
		} catch (Exception e) {
			addErrorMessage();
		}
		return null;
	}

	/**
	 * Delete command.
	 */
	public void deleteCommand() {

		try {
			if (commander.deleteCommand(commandId)) {
				addSucessMessage("Sucess");
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
				retrieveCommandsList();
			}
		} catch (Exception e) {
			addErrorMessage();
		}
	}

	/**
	 * Inits the.
	 */
	@PostConstruct
	public void init() {

		try {
			commandsCollection.addAll(commander.listCommands());
		} catch (Exception e) {
			addErrorMessage();
		}

	}

	/**
	 * Adds the sucess message.
	 *
	 * @param message the message
	 */
	protected void addSucessMessage(String message) {

		FacesContext fc = FacesContext.getCurrentInstance();
		fc.addMessage(null, new FacesMessage(message));
	}

	/**
	 * Adds the error message.
	 */
	protected void addErrorMessage() {
		FacesContext fc = FacesContext.getCurrentInstance();
		fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error getting commands", null));
	}

	/**
	 * Instantiates a new list commands beans.
	 */
	public ListCommandsBeans() {

	}

}
