/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPresentation;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import javax.annotation.PostConstruct;
import javax.enterprise.context.SessionScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import org.primefaces.event.RowEditEvent;
import jtrackingBusiness.IManagementBusiness;
import jtrackingPersistence.CommunicationType;
import jtrackingPersistence.DeviceJPA;

/**
 * The Class ListDevicesBean.
 */
@Named("devices")
@SessionScoped
public class ListDevicesBean implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -164825335845644230L;

	/** The management. */
	@Inject
	private IManagementBusiness management;

	/** The devices collection. */
	private Collection<DeviceJPA> devicesCollection = new ArrayList<>();

	/** The communications types collection. */
	private Collection<CommunicationType> communicationsTypesCollection = new ArrayList<>();


	/** The device id. */
	private Integer deviceId;
	
	/** The communication type. */
	private CommunicationType communicationType;
	
	/**
	 * Gets the devices collection.
	 *
	 * @return the devices collection
	 */
	public Collection<DeviceJPA> getDevicesCollection() {
		return devicesCollection;
	}

	/**
	 * Sets the devices collection.
	 *
	 * @param devicesCollection the new devices collection
	 */
	public void setDevicesCollection(Collection<DeviceJPA> devicesCollection) {
		this.devicesCollection = devicesCollection;
		
	}

	/**
	 * Gets the device id.
	 *
	 * @return the device id
	 */
	public Integer getDeviceId() {
		return deviceId;
	}

	/**
	 * Sets the device id.
	 *
	 * @param deviceId the new device id
	 */
	public void setDeviceId(Integer deviceId) {
		this.deviceId = deviceId;
	}
	
	/**
	 * Inits the.
	 */
	@PostConstruct
	public void init() {

		communicationsTypesCollection.addAll(Arrays.asList(CommunicationType.values()));
	}

	

	/**
	 * Gets the devices list.
	 *
	 * @return the devices list
	 */
	public void getDevicesList() {
		try {
			devicesCollection.clear();
			devicesCollection.addAll(management.getDevices());
			FacesContext.getCurrentInstance().getExternalContext().redirect("devices.xhtml");
			
		} catch (Exception e) {
			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error while getting devices", null));
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
		}

	}

	/**
	 * Delete device.
	 */
	public void deleteDevice() {

		try {
			if(management.deleteDevice(deviceId)) {
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null, new FacesMessage("Sucess"));
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
				getDevicesList();
			}
			else
			{
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Imposible to delete " + deviceId, null));
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);

			}
		} catch (Exception e) {
			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Imposible to delete " + deviceId, null));
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);

		}
	}

	/**
	 * Gets the communications types collection.
	 *
	 * @return the communications types collection
	 */
	public Collection<CommunicationType> getCommunicationsTypesCollection() {
		return communicationsTypesCollection;
	}

	/**
	 * Sets the communications types collection.
	 *
	 * @param communicationsTypesCollection the new communications types collection
	 */
	public void setCommunicationsTypesCollection(Collection<CommunicationType> communicationsTypesCollection) {
		this.communicationsTypesCollection = communicationsTypesCollection;
	}

	/**
	 * Gets the communication type.
	 *
	 * @return the communication type
	 */
	public CommunicationType getCommunicationType() {
		return communicationType;
	}

	/**
	 * Sets the communication type.
	 *
	 * @param communicationType the new communication type
	 */
	public void setCommunicationType(CommunicationType communicationType) {
		this.communicationType = communicationType;
	}

	
	/**
	 * On edit.
	 *
	 * @param event the event
	 */
	public void onEdit(RowEditEvent event) {

		DeviceJPA dev = (DeviceJPA) event.getObject();
		try {		
			if(management.updateDevice(dev.getId(), dev.getModel(), dev.getSerialNumber(), dev.getFirmwareVersion(), communicationType))
			{
				
			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null, new FacesMessage("Sucess"));
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			}
			else
			{
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null,
						new FacesMessage(FacesMessage.SEVERITY_ERROR, "error while updating", null));
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);

			}
		}
			catch (Exception e) {
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error while updating", null));
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			}
		finally {
			this.communicationType = null;
		}
		getDevicesList();
	}
		
		
		/**
		 * On cancel.
		 *
		 * @param event the event
		 * @return the string
		 */
	public String onCancel(RowEditEvent event) {

		FacesContext fc = FacesContext.getCurrentInstance();
		fc.addMessage(null, new FacesMessage("Action cancelled by user"));
		return null;
	}
	
	/**
	 * Instantiates a new list devices bean.
	 */
	public ListDevicesBean() {

	}

}
