/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPresentation;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import javax.enterprise.context.SessionScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import org.primefaces.event.RowEditEvent;
import jtrackingBusiness.IManagementBusiness;
import jtrackingPersistence.MissionJPA;

/**
 * The Class ListMissionsBean.
 */
@Named("missions")
@SessionScoped
public class ListMissionsBean implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -1804391618990085983L;

	/** The management. */
	@Inject
	private IManagementBusiness management;

	/** The missions collection. */
	private Collection<MissionJPA> missionsCollection = new ArrayList<>();

	/** The mission id. */
	private String missionId;

	/**
	 * Gets the missions collection.
	 *
	 * @return the missions collection
	 */
	public Collection<MissionJPA> getMissionsCollection() {
		return missionsCollection;
	}

	/**
	 * Sets the missions collection.
	 *
	 * @param missionsCollection the new missions collection
	 */
	public void setMissionsCollection(Collection<MissionJPA> missionsCollection) {
		this.missionsCollection = missionsCollection;
	}

	/**
	 * Gets the mission id.
	 *
	 * @return the mission id
	 */
	public String getMissionId() {
		return missionId;
	}

	/**
	 * Sets the mission id.
	 *
	 * @param missionId the new mission id
	 */
	public void setMissionId(String missionId) {
		this.missionId = missionId;
	}

	/**
	 * Gets the missions list.
	 *
	 * @return the missions list
	 */
	public void getMissionsList() {
		try {
			missionsCollection.clear();
			missionsCollection.addAll(management.getMissions());
			FacesContext.getCurrentInstance().getExternalContext().redirect("missions.xhtml");
		} catch (Exception e) {
			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error while getting missions", null));
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
		}

		
	}

	/**
	 * Delete mission.
	 */
	public void deleteMission() {

		try {
			if (management.deleteMission(missionId)) {
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null, new FacesMessage("Sucess"));
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
				getMissionsList();
			}
			else
			{
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null,
						new FacesMessage(FacesMessage.SEVERITY_ERROR, "Imposible to delete " + missionId, null));
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			}
		} catch (Exception e) {
			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "Imposible to delete " + missionId, null));
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);

		}
	}

	/**
	 * On edit.
	 *
	 * @param event the event
	 */
	public void onEdit(RowEditEvent event) {

		MissionJPA mission = (MissionJPA) event.getObject();
		try {
			if (mission.getEndDate() != null) {
				if (mission.getEndDate().after(mission.getStartDate())) {
					if (management.updateMission(mission.getId(), mission.getDescription(), mission.getStartDate(),
							mission.getEndDate())) {
						FacesContext fc = FacesContext.getCurrentInstance();
						fc.addMessage(null, new FacesMessage("Sucess"));
						FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
					}

				} else {

					FacesContext fc = FacesContext.getCurrentInstance();
					fc.addMessage(null,
							new FacesMessage(FacesMessage.SEVERITY_ERROR, "End date before start date", null));
					FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);

				}

			} else {
				if (management.updateMission(mission.getId(), mission.getDescription(), mission.getStartDate(),
						mission.getEndDate())) {
					FacesContext fc = FacesContext.getCurrentInstance();
					fc.addMessage(null, new FacesMessage("Sucess"));
					FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
				}

			}

		}

		catch (Exception e) {

			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error while updating", null));
		}
		getMissionsList();

	}

	/**
	 * On cancel.
	 *
	 * @param event the event
	 * @return the string
	 */
	public String onCancel(RowEditEvent event) {

		FacesContext fc = FacesContext.getCurrentInstance();
		fc.addMessage(null, new FacesMessage("Action cancelled by user"));
		return null;
	}

}
