/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPresentation;

import java.io.Serializable;
import java.util.Collection;

import javax.annotation.PostConstruct;
import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.inject.Named;

import org.primefaces.model.map.DefaultMapModel;
import org.primefaces.model.map.LatLng;
import org.primefaces.model.map.MapModel;
import org.primefaces.model.map.Marker;
import org.primefaces.model.map.Polyline;

import jtrackingPersistence.PositionJPA;

/**
 * The Class MappingBean.
 */
@Named("mapping")
@RequestScoped
public class MappingBean implements Serializable{
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -6991723724835654417L;
	
	/** The positionsbean. */
	@Inject
	private PositionsBean positionsbean;

	/** The simple model. */
	private MapModel simpleModel;
	
	/** The positionscollection. */
	private Collection<PositionJPA> positionscollection;
	
	/** The center. */
	private String center;
	
	/** The zoom. */
	private int zoom;
	
	/** The poly line. */
	private Polyline polyLine;
	
	/** The lat. */
	private Double lat;
	
	/** The lon. */
	private Double lon;
	
	/** The color. */
	private String color;
	
	/** The interval. */
	private int interval;
	
	
	
	/**
	 * Gets the simple model.
	 *
	 * @return the simple model
	 */
	public MapModel getSimpleModel() {
		return simpleModel;
	}

	/**
	 * Sets the simple model.
	 *
	 * @param simpleModel the new simple model
	 */
	public void setSimpleModel(MapModel simpleModel) {
		this.simpleModel = simpleModel;
	}

	/**
	 * Gets the positionscollection.
	 *
	 * @return the positionscollection
	 */
	public Collection<PositionJPA> getPositionscollection() {
		return positionscollection;
	}

	/**
	 * Sets the positionscollection.
	 *
	 * @param positionscollection the new positionscollection
	 */
	public void setPositionscollection(Collection<PositionJPA> positionscollection) {
		this.positionscollection = positionscollection;
	}

	/**
	 * Gets the center.
	 *
	 * @return the center
	 */
	public String getCenter() {
		return center;
	}

	/**
	 * Sets the center.
	 *
	 * @param center the new center
	 */
	public void setCenter(String center) {
		this.center = center;
	}

	/**
	 * Gets the zoom.
	 *
	 * @return the zoom
	 */
	public int getZoom() {
		return zoom;
	}

	/**
	 * Sets the zoom.
	 *
	 * @param zoom the new zoom
	 */
	public void setZoom(int zoom) {
		this.zoom = zoom;
	}

	/**
	 * Gets the poly line.
	 *
	 * @return the poly line
	 */
	public Polyline getPolyLine() {
		return polyLine;
	}

	/**
	 * Sets the poly line.
	 *
	 * @param polyLine the new poly line
	 */
	public void setPolyLine(Polyline polyLine) {
		this.polyLine = polyLine;
	}

	/**
	 * Gets the lat.
	 *
	 * @return the lat
	 */
	public Double getLat() {
		return lat;
	}

	/**
	 * Sets the lat.
	 *
	 * @param lat the new lat
	 */
	public void setLat(Double lat) {
		this.lat = lat;
	}

	/**
	 * Gets the lon.
	 *
	 * @return the lon
	 */
	public Double getLon() {
		return lon;
	}

	/**
	 * Sets the lon.
	 *
	 * @param lon the new lon
	 */
	public void setLon(Double lon) {
		this.lon = lon;
	}

	/**
	 * Gets the color.
	 *
	 * @return the color
	 */
	public String getColor() {
		return color;
	}

	/**
	 * Sets the color.
	 *
	 * @param color the new color
	 */
	public void setColor(String color) {
		StringBuilder sb = new StringBuilder();
		sb.append("#");
		sb.append(color);
		this.color = sb.toString();
	}

	/**
	 * Gets the interval.
	 *
	 * @return the interval
	 */
	public int getInterval() {
		return interval;
	}

	/**
	 * Sets the interval.
	 *
	 * @param interval the new interval
	 */
	public void setInterval(int interval) {
		this.interval = interval;
	}

	/**
	 * Clear map.
	 *
	 * @return the string
	 */
	public String clearMap() {
		setSimpleModel(new DefaultMapModel());
		return "mapping";
	}

	/**
	 * Inits the.
	 */
	@PostConstruct
	public void init() {
		this.center = "41.406422, 2.194394";
		this.zoom = 11;
		this.interval = 10;
		this.simpleModel= new DefaultMapModel();
		this.polyLine  = new Polyline();
	}
	

	/**
	 * Paint positions.
	 *
	 * @return the string
	 */
	public String paintPositions()
	{
		positionsbean.retrievePositionsTable();
		positionscollection = positionsbean.getPositionsTable();
	
		int i = 0;
		for(PositionJPA pos: positionscollection)
		{
			StringBuilder title = new StringBuilder();
			
			Marker mkr = new Marker(new LatLng(pos.getLatitude(), pos.getLongitude()), pos.getId().toString(), null, null);
			if(i == 0)
			{
				mkr.setIcon("http://maps.google.com/intl/en_us/mapfiles/ms/micons/green-dot.png");
			}
			else if(i == positionscollection.size()-1)
			{
				mkr.setIcon("http://maps.google.com/intl/en_us/mapfiles/ms/micons/red-dot.png");
			}
			else
			{
				mkr.setIcon("http://maps.google.com/intl/en_us/mapfiles/ms/micons/blue-dot.png");
			}
			
			mkr.setClickable(true);
			mkr.setDraggable(true);
			title.append(String.valueOf(pos.getId()));
			title.append("  ");
			title.append(String.valueOf(pos.getPositionTimeStamp()));
			title.append("  ");
			title.append(String.valueOf((pos.getLatitude())));
			title.append("  ");
			title.append(String.valueOf((pos.getLongitude())));
			mkr.setTitle(title.toString());
			simpleModel.addOverlay(mkr);
			polyLine.getPaths().add(new LatLng(pos.getLatitude(), pos.getLongitude()));
			
			String centro = mkr.getLatlng().toString();
			centro = centro.replace("Lat:", "");
			centro = centro.replace("Lng:", "");
			this.center= centro;
			
			i++;
			
		}
		

		simpleModel.addOverlay(polyLine);
		polyLine.setStrokeWeight(5);
		polyLine.setStrokeColor(getColor());
        polyLine.setStrokeOpacity(1.0);
          
		return null;
	}
	
	/**
	 * Gets the real time.
	 *
	 * @return the real time
	 */
	public String getRealTime() {
		positionsbean.getRealTime();
		positionscollection = positionsbean.getPositions();
	
		int i = 0;
		for(PositionJPA pos: positionscollection)
		{
			StringBuilder title = new StringBuilder();
			
			Marker mkr = new Marker(new LatLng(pos.getLatitude(), pos.getLongitude()), pos.getId().toString(), null, null);
			if(i == 0)
			{
				String centro = mkr.getLatlng().toString();
				centro = centro.replace("Lat:", "");
				centro = centro.replace("Lng:", "");
				this.center= centro;
				mkr.setIcon("http://maps.google.com/intl/en_us/mapfiles/ms/micons/red-dot.png");
			}
			else
			{
				mkr.setIcon("http://maps.google.com/intl/en_us/mapfiles/ms/micons/blue-dot.png");
			}
			
			mkr.setClickable(true);
			mkr.setDraggable(true);
			title.append(String.valueOf(pos.getId()));
			title.append("  ");
			title.append(String.valueOf(pos.getPositionTimeStamp()));
			title.append("  ");
			title.append(String.valueOf((pos.getLatitude())));
			title.append("  ");
			title.append(String.valueOf((pos.getLongitude())));
			mkr.setTitle(title.toString());
			simpleModel.addOverlay(mkr);
			polyLine.getPaths().add(new LatLng(pos.getLatitude(), pos.getLongitude()));
			
			
			
			i++;
			
		}
		

		simpleModel.addOverlay(polyLine);
		polyLine.setStrokeWeight(5);
		polyLine.setStrokeColor(getColor());
        polyLine.setStrokeOpacity(1.0);

		
		return null;
	}

	
	/**
	 * Instantiates a new mapping bean.
	 */
	public MappingBean() {
	
	}

}
