/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPresentation;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import javax.annotation.PostConstruct;
import javax.enterprise.context.RequestScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.inject.Inject;
import javax.inject.Named;
import jtrackingBusiness.ICommandBusiness;
import jtrackingBusiness.IManagementBusiness;
import jtrackingPersistence.DeviceJPA;

/**
 * The Class SendCommandBean.
 */
@Named("command")
@RequestScoped
public class SendCommandBean implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -5259030279235943703L;

	/** The management. */
	@Inject
	private IManagementBusiness management;

	/** The commander. */
	@Inject
	private ICommandBusiness commander;

	/** The devices collection. */
	private Collection<DeviceJPA> devicesCollection = new ArrayList<>();

	/** The devices id collection. */
	private Collection<Integer> devicesIdCollection = new ArrayList<>();

	/** The device id. */
	private Integer deviceId;

	/**
	 * Gets the devices collection.
	 *
	 * @return the devices collection
	 */
	public Collection<DeviceJPA> getDevicesCollection() {
		return devicesCollection;
	}

	/**
	 * Sets the devices collection.
	 *
	 * @param devicesCollection the new devices collection
	 */
	public void setDevicesCollection(Collection<DeviceJPA> devicesCollection) {
		this.devicesCollection = devicesCollection;
	}

	/**
	 * Gets the devices id collection.
	 *
	 * @return the devices id collection
	 */
	public Collection<Integer> getDevicesIdCollection() {
		return devicesIdCollection;
	}

	/**
	 * Sets the devices id collection.
	 *
	 * @param devicesIdCollection the new devices id collection
	 */
	public void setDevicesIdCollection(Collection<Integer> devicesIdCollection) {
		this.devicesIdCollection = devicesIdCollection;
	}

	/**
	 * Gets the device id.
	 *
	 * @return the device id
	 */
	public Integer getDeviceId() {
		return deviceId;
	}

	/**
	 * Sets the device id.
	 *
	 * @param deviceId the new device id
	 */
	public void setDeviceId(Integer deviceId) {
		this.deviceId = deviceId;
	}

	/**
	 * Inits the.
	 */
	@PostConstruct
	public void init() {

		devicesCollection.addAll(management.getDevices());
		for (DeviceJPA dev : devicesCollection) {
			devicesIdCollection.add(dev.getId());
		}

	}

	/**
	 * Send.
	 *
	 * @param evt the evt
	 */
	public void send(ActionEvent evt) {

		String actionId = "";
		actionId = evt.getComponent().getClientId().toString();
		switch (actionId) {
		case "battery":
			try {
				if (commander.requestBatteyLevel(deviceId)) {
					addSucessMessage("Sucess: battery request queued");
				} else {
					addErrorMessage();

				}
			} catch (Exception e) {
				addErrorMessage();
			}

			break;

		case "signal":
			try {
				if (commander.requestGsmSignal(deviceId)) {
					addSucessMessage("Sucess: GSM signal request queued");
				} else {
					addErrorMessage();

				}
			} catch (Exception e) {
				addErrorMessage();
			}
			break;

		case "status":
			try {
				if (commander.requestStatus(deviceId)) {
					addSucessMessage("Sucess: status request queued");
				} else {
					addErrorMessage();
				}
			} catch (Exception e) {
				addErrorMessage();
			}
			break;

		case "start":
			try {
				if (commander.startSendPositions(deviceId)) {
					addSucessMessage("Sucess: sending request queued");
					;
				} else {
					addErrorMessage();
				}
			} catch (Exception e) {
				addErrorMessage();
			}
			break;

		case "stop":
			try {
				if (commander.stopSendPositions(deviceId)) {
					addSucessMessage("Sucess: GSM signal request queued");
				} else {
					addErrorMessage();
				}
			} catch (Exception e) {
				addErrorMessage();
			}
			break;

		default:
			addErrorMessage();
			break;
		}
		actionId = "";
	}

	/**
	 * Adds the sucess message.
	 *
	 * @param message the message
	 */
	protected void addSucessMessage(String message) {

		FacesContext fc = FacesContext.getCurrentInstance();
		fc.addMessage(null, new FacesMessage(message));
	}

	/**
	 * Adds the error message.
	 */
	protected void addErrorMessage() {
		FacesContext fc = FacesContext.getCurrentInstance();
		fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error requesting", null));
	}

	/**
	 * Instantiates a new send command bean.
	 */
	public SendCommandBean() {

	}

}
