/*
 * Proyecto J-Tracking
 * Trabajo de Fin de Grado UOC 2017
 * Alumno: Rafael Fernández Lorenzo
 * Consultor: Vicenç Font Sagrista
 */
package jtrackingPresentation;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import javax.enterprise.context.SessionScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import jtrackingBusiness.IManagementBusiness;
import jtrackingPersistence.DeviceJPA;
import jtrackingPersistence.PhoneLineJPA;

/**
 * The Class UpdatePhoneLineBean.
 */
@Named("updatephoneline")
@SessionScoped
public class UpdatePhoneLineBean implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -21596011821706187L;

	/** The management. */
	@Inject
	private IManagementBusiness management;
	
	/** The list phone lines bean. */
	@Inject
	private ListPhoneLinesBean listPhoneLinesBean;
	
	/** The phone line. */
	private PhoneLineJPA phoneLine;
	
	/** The number. */
	private String number;
	
	/** The icc. */
	private String ICC;
	
	/** The pin. */
	private String PIN;
	
	/** The puk. */
	private String PUK;
	
	/** The company. */
	private String company;
	
	/** The holder. */
	private String holder;
	
	/** The device id. */
	private Integer deviceId;
	
	/** The devices collection. */
	private Collection<DeviceJPA> devicesCollection = new ArrayList<>();
	
	/** The devices id collection. */
	private Collection<Integer> devicesIdCollection = new ArrayList<>();
		
	/**
	 * Gets the phone line.
	 *
	 * @return the phone line
	 */
	public PhoneLineJPA getPhoneLine() {
		return phoneLine;
	}


	/**
	 * Sets the phone line.
	 *
	 * @param phoneLine the new phone line
	 */
	public void setPhoneLine(PhoneLineJPA phoneLine) {
		this.phoneLine = phoneLine;
	}


	/**
	 * Gets the number.
	 *
	 * @return the number
	 */
	public String getNumber() {
		return number;
	}


	/**
	 * Sets the number.
	 *
	 * @param number the new number
	 */
	public void setNumber(String number) {
		this.number = number;
	}


	/**
	 * Gets the icc.
	 *
	 * @return the icc
	 */
	public String getICC() {
		return ICC;
	}


	/**
	 * Sets the icc.
	 *
	 * @param iCC the new icc
	 */
	public void setICC(String iCC) {
		ICC = iCC;
	}


	/**
	 * Gets the pin.
	 *
	 * @return the pin
	 */
	public String getPIN() {
		return PIN;
	}


	/**
	 * Sets the pin.
	 *
	 * @param pIN the new pin
	 */
	public void setPIN(String pIN) {
		PIN = pIN;
	}


	/**
	 * Gets the puk.
	 *
	 * @return the puk
	 */
	public String getPUK() {
		return PUK;
	}


	/**
	 * Sets the puk.
	 *
	 * @param pUK the new puk
	 */
	public void setPUK(String pUK) {
		PUK = pUK;
	}


	/**
	 * Gets the company.
	 *
	 * @return the company
	 */
	public String getCompany() {
		return company;
	}


	/**
	 * Sets the company.
	 *
	 * @param company the new company
	 */
	public void setCompany(String company) {
		this.company = company;
	}


	/**
	 * Gets the holder.
	 *
	 * @return the holder
	 */
	public String getHolder() {
		return holder;
	}


	/**
	 * Sets the holder.
	 *
	 * @param holder the new holder
	 */
	public void setHolder(String holder) {
		this.holder = holder;
	}


	/**
	 * Gets the device id.
	 *
	 * @return the device id
	 */
	public Integer getDeviceId() {
		return deviceId;
	}


	/**
	 * Sets the device id.
	 *
	 * @param deviceId the new device id
	 */
	public void setDeviceId(Integer deviceId) {
		this.deviceId = deviceId;
	}


	/**
	 * Gets the devices collection.
	 *
	 * @return the devices collection
	 */
	public Collection<DeviceJPA> getDevicesCollection() {
		return devicesCollection;
	}


	/**
	 * Sets the devices collection.
	 *
	 * @param devicesCollection the new devices collection
	 */
	public void setDevicesCollection(Collection<DeviceJPA> devicesCollection) {
		this.devicesCollection = devicesCollection;
	}


	/**
	 * Gets the devices id collection.
	 *
	 * @return the devices id collection
	 */
	public Collection<Integer> getDevicesIdCollection() {
		return devicesIdCollection;
	}


	/**
	 * Sets the devices id collection.
	 *
	 * @param devicesIdCollection the new devices id collection
	 */
	public void setDevicesIdCollection(Collection<Integer> devicesIdCollection) {
		this.devicesIdCollection = devicesIdCollection;
	}


	/**
	 * Show phone line data.
	 *
	 * @return the string
	 */
	public String showPhoneLineData() {
		
		this.number = phoneLine.getPhoneNumber();
		this.ICC= phoneLine.getICC();
		this.PIN = phoneLine.getPIN();
		this.PUK = phoneLine.getPUK();
		this.company = phoneLine.getCompany();
		this.holder = phoneLine.getHolder();
		this.deviceId = null;
		if(phoneLine.getDevice() != null)
		{
			this.deviceId = phoneLine.getDevice().getId();
		}
		try {
			devicesCollection.clear();
			devicesIdCollection.clear();		
			devicesCollection.addAll(management.getDevices());
			for(DeviceJPA dev: devicesCollection)
			{
				if(dev.getPhoneLine() == null)
				{
					devicesIdCollection.add(dev.getId());
				}
				
			}
			
		} catch (Exception e) {
			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null,  new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error while getting phone line data", null));
		}
		return "updatephoneline.xhtml?faces-redirect=true";
	}
	
	
	/**
	 * Update phone line.
	 */
	public void updatePhoneLine() {
		
		try {
			if(management.updatePhoneLine(number, ICC, PIN, PUK, company, holder, deviceId)) {
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null, new FacesMessage("Sucess", "Phone line updated"));
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			}
			else {
				FacesContext fc = FacesContext.getCurrentInstance();
				fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error to update ", null));
				FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
			}
			
		} catch (Exception e) {
			FacesContext fc = FacesContext.getCurrentInstance();
			fc.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error to update ", null));
			FacesContext.getCurrentInstance().getExternalContext().getFlash().setKeepMessages(true);
		}
		listPhoneLinesBean.getPhoneLinesList();
		
	}
	
	/**
	 * Instantiates a new update phone line bean.
	 */
	public UpdatePhoneLineBean() {
		
	}

}
